@@ -14,17 +14,32 @@ package Apache::Clean;
 use 5.004;
 use mod_perl 1.21;
 use Apache::Constants qw( OK DECLINED );
+use Apache;
 use Apache::File;
 use Apache::Log;
 use HTML::Clean;
 use strict;
 
-$Apache::Clean::VERSION = '0.03';
+$Apache::Clean::VERSION = '0.04';
 
 # set debug level
 #  0 - messages at info or debug log levels
 #  1 - verbose output at info or debug log levels
-$Apache::Clean::DEBUG = 0;
+$Apache::Clean::DEBUG = 1;
+
+# Get the package modification time...
+(my $package = __PACKAGE__) =~ s!::!/!g;
+my $package_mtime = (stat $INC{"$package.pm"})[9];
+
+# ...and when httpd.conf was last modified
+my $conf_mtime = (stat Apache->server_root_relative('conf/httpd.conf'))[9];
+
+# When the server is restarted we need to
+# make sure we recognize config file changes and propigate
+# them to the client to clear the client cache if necessary.
+Apache->server->register_cleanup(sub {
+  $conf_mtime = (stat Apache->server_root_relative('conf/httpd.conf'))[9];
+});
 
 sub handler {
 #---------------------------------------------------------------------
@@ -35,7 +50,7 @@ sub handler {
 
   my $log          = $r->server->log;
 
-  my $fh           = undef;
+  my ($fh, $cache) = ();
 
 #---------------------------------------------------------------------
 # do some preliminary stuff...
@@ -43,7 +58,7 @@ sub handler {
 
   $log->info("Using Apache::Clean");
 
-  unless ($r->content_type eq 'text/html') {
+  unless ($r->content_type =~ m!text/html!i) {
     $log->info("\trequest is not for an html document - skipping...")
        if $Apache::Clean::DEBUG;
     $log->info("Exiting Apache::Clean");
@@ -87,6 +102,28 @@ sub handler {
   
       return DECLINED;
     }
+
+    # since we're essentially sending a static file
+    # we can set cache headers properly based on the
+    # file itself - although we're modifying the 
+    # content the meaning of the content doesn't
+    # change unless it:
+    #   changes on disk
+    #   this package is modified
+    #   our httpd.conf options have changed
+
+    # however, in the interests of back compatibility, make
+    # proper cache behavior an option
+    $cache = lc $r->dir_config('CleanCache') || 'on';
+
+    if ($cache eq 'on') {
+      # set what we can from here, more later...
+      $r->update_mtime($package_mtime);
+      $r->update_mtime((stat $r->finfo)[9]);
+      $r->update_mtime($conf_mtime);
+      $r->set_last_modified;
+      $r->set_etag;
+    }
   }
 
 #---------------------------------------------------------------------
@@ -111,8 +148,24 @@ sub handler {
 # print the clean results
 #---------------------------------------------------------------------
 
-  # Send the crisp, clean data.
-  $r->send_http_header('text/html');
+  if ($cache eq 'on') {
+    # we needed to clean the data first before we
+    # could find the length
+    $r->set_content_length(length ${$h->data});
+
+    # only send the file if it meets cache criteria
+    if ((my $status = $r->meets_conditions) == OK) {
+      $r->send_http_header('text/html');
+    }
+    else {
+      return $status;
+    }
+  }
+  else {
+    # else we just send a header
+    $r->send_http_header('text/html');
+  }
+
   print ${$h->data};
 
 #---------------------------------------------------------------------
@@ -136,6 +189,8 @@ Apache::Clean - mod_perl interface into HTML::Clean
 
 httpd.conf:
 
+ PerlModule Apache::Clean
+
  <Location /clean>
     SetHandler perl-script
     PerlHandler Apache::Clean
@@ -144,6 +199,8 @@ httpd.conf:
 
     PerlSetVar  CleanOption shortertags
     PerlAddVar  CleanOption whitespace
+
+    PerlSetVar  CleanCache On
  </Location>  
 
 Apache::Clean is Filter aware, meaning that it can be used within
@@ -161,14 +218,64 @@ Apache::Clean uses HTML::Clean to tidy up large, messy HTML, saving
 bandwidth.  It is particularly useful with Apache::Compress for 
 ultimate savings.
 
-The only current configuration directive is CleanLevel, which defaults
-to 3.  Apache::Clean will only tidy up whitespace (via $h->strip) and
-will not perform other options of HTML::Clean (such as browser
-compatibility).  See the HTML::Clean manpage for details.
-
 Only documents with a content type of "text/html" are affected - all
 others are passed through unaltered.
 
+=head1 OPTIONS
+
+Apache::Clean supports few options, most of which are based on
+options from HTML::Clean.  Apache::Clean will only tidy up whitespace 
+(via $h->strip) and will not perform other options of HTML::Clean
+(such as browser compatibility).  See the HTML::Clean manpage for 
+details.
+
+=over 4
+
+=item CleanLevel
+
+sets the clean level, which is passed to the level() method
+in HTML::Clean.
+
+  PerlSetVar CleanLevel 9
+
+CleanLevel defaults to 3.
+
+=item CleanOption
+
+specifies the set of options which are passed to the options()
+method in HTML::Clean.
+
+  PerlAddVar CleanOption shortertags
+  PerlSetVar CleanOption whitespace
+
+CleanOption has do default.
+
+=item CleanCache
+
+sets the behavior of Apache::Clean in regards to proper
+cache header behavior.  this option is only meaningful
+when Apache::Clean is _not_ part of an Apache::Filter
+chain.
+
+mainly, CleanCache On enables Apache::Clean to
+set the Last-Modified, Content-Length, and Etag headers,
+as well as allowing it do decide whether a 304 response
+is allowed.  See recipe 6.6 in the mod_perl Developer's
+Cookbook for a more detailed discussion on handling
+conditional and cache-based headers - the code is
+practically identical to what you will find there.
+
+The basic idea here is that although Apache::Clean is
+dynamically manipulating the content of the requested
+resource, the meaning of the document has not changed
+just because <strong> was changed to <b>.  If you
+disagree with this assessment you can set CleanCache to
+Off.
+
+CleanCache defaults to On.
+
+=back
+
 =head1 NOTES
 
 Verbose debugging is enabled by setting $Apache::Clean::DEBUG=1
@@ -181,7 +288,7 @@ PERL_FILE_API=1, and maybe other hooks to function properly.
 
 =head1 FEATURES/BUGS
 
-  No known bugs or features at this time...
+No known bugs or features at this time...
 
 =head1 SEE ALSO
 
